"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = committable;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _pathHelpers = require("../helpers/pathHelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 * Returns an enhanced component that detects that a change to the value of the base component has
 * been committed, and executes a callback with the new value. A change is considered to be
 * committed when the base component loses focus, or the enter key is pressed while it has focus.
 * @param {string|function} BaseComponent - The component to enhance. This component must accept
 * onBlur and onKeyPress props.
 * @returns {function} The enhanced component, which accepts an onCommit prop containing a function
 * to be called with the path to the input, and its new value.
 */
function committable(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
  const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
    // Until then, propTypes need to be hoisted from the base component.
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ...BaseComponent.propTypes,
    commitUnchanged: _propTypes.default.bool,
    readOnly: _propTypes.default.bool,
    onBlur: _propTypes.default.func,
    onCommit: _propTypes.default.func,
    onKeyPress: _propTypes.default.func
  };
  const defaultProps = {
    commitUnchanged: false,
    readOnly: undefined,
    onBlur: undefined,
    onCommit: undefined,
    onKeyPress: undefined
  };

  class Committable extends _react.Component {
    constructor(props) {
      super(props);
      this.handleBlur = this.handleBlur.bind(this);
      this.handleKeyPress = this.handleKeyPress.bind(this);
    }

    handleBlur(event) {
      const {
        onBlur,
        readOnly
      } = this.props;

      if (onBlur) {
        onBlur(event);
      }

      if (!readOnly) {
        this.commit(event.target.value);
      }
    }

    handleKeyPress(event) {
      if (event.key === 'Enter') {
        this.commit(event.target.value);
      }

      const {
        onKeyPress
      } = this.props;

      if (onKeyPress) {
        onKeyPress(event);
      }
    }

    commit(value) {
      const {
        commitUnchanged,
        value: initialValue,
        onCommit
      } = this.props;

      if (onCommit && ((value || initialValue) && value !== initialValue || commitUnchanged)) {
        onCommit((0, _pathHelpers.getPath)(this.props), value);
      }
    }

    render() {
      const {
        commitUnchanged,
        onCommit,
        onKeyPress,
        ...remainingProps
      } = this.props;
      return /*#__PURE__*/_react.default.createElement(BaseComponent // eslint-disable-next-line react/jsx-props-no-spreading
      , _extends({}, remainingProps, {
        onBlur: this.handleBlur,
        onKeyPress: this.handleKeyPress
      }));
    }

  }

  Committable.propTypes = propTypes;
  Committable.defaultProps = defaultProps;
  Committable.displayName = `committable(${baseComponentName})`;
  return Committable;
}