"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = labelable;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _normalizeLabel = _interopRequireDefault(require("../helpers/normalizeLabel"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Makes an input component labelable. Returns an enhanced component that accepts a label prop. If
 * a label is supplied, the base component is wrapped, and rendered in the wrapper along with the
 * label. If the label is a string, it is wrapped in a Label; otherwise, it is rendered as given.
 * If no label is supplied, the base component is returned unchanged. A msgkey prop is also
 * accepted, but has no effect. It may be used by preprocessors to generate a label.
 * @param {string|function} BaseComponent - The component to enhance.
 * @returns {function} The enhanced component.
 */
function labelable(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
  const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
    // Until then, propTypes need to be hoisted from the base component.
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ...BaseComponent.propTypes,
    label: _propTypes.default.node,
    msgkey: _propTypes.default.string,
    readOnly: _propTypes.default.bool,
    required: _propTypes.default.bool
  };
  const defaultProps = {
    label: undefined,
    msgkey: undefined,
    readOnly: undefined,
    required: false
  };

  function Labelable(props) {
    const {
      label,
      // eslint-disable-next-line no-unused-vars
      msgkey,
      required,
      ...remainingProps
    } = props;
    const {
      readOnly
    } = props;
    const normalizedLabel = (0, _normalizeLabel.default)(label, {
      required,
      readOnly
    });

    const baseComponent =
    /*#__PURE__*/
    // eslint-disable-next-line react/jsx-props-no-spreading
    _react.default.createElement(BaseComponent, remainingProps);

    if (!normalizedLabel) {
      return baseComponent;
    }

    return /*#__PURE__*/_react.default.createElement("div", null, normalizedLabel, baseComponent);
  }

  Labelable.propTypes = propTypes;
  Labelable.defaultProps = defaultProps;
  Labelable.displayName = `labelable(${baseComponentName})`;
  return Labelable;
}